"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.K8sApiSchemaFetcher = void 0;
const k8s_api_schema_registry_1 = require("./k8s-api-schema-registry");
const k8s_super_client_1 = require("k8s-super-client");
const spinner_1 = require("../screen/spinner");
class K8sApiSchemaFetcher {
    constructor(logger) {
        this._logger = logger.sublogger('K8sApiSchemaFetcher');
    }
    fetchRemote(k8sConnector) {
        return __awaiter(this, void 0, void 0, function* () {
            const spinner = (0, spinner_1.spinOperation)('Extracting K8s API Schema...');
            const result = {
                success: false,
                targetVersion: null,
                selectedVersion: null,
                found: false,
                foundExact: false,
                k8sJsonSchema: null,
            };
            return Promise.resolve()
                .then(() => {
                const client = k8sConnector.client;
                if (!client) {
                    result.error = 'Could not connect to Kubernetes cluster';
                    return;
                }
                return client.openAPI.queryClusterVersion()
                    .then(version => {
                    this._logger.info("Cluster version: %s", version);
                    return client.openAPI.queryApiSpecs()
                        .then(k8sOpenApiSpecs => {
                        spinner.complete('K8s API Schema Fetched.');
                        const converter = new k8s_super_client_1.K8sOpenApiSpecToJsonSchemaConverter(this._logger, k8sOpenApiSpecs);
                        const k8sJsonSchema = converter.convert();
                        result.success = true;
                        result.selectedVersion = version;
                        result.found = true;
                        result.foundExact = true;
                        result.k8sJsonSchema = k8sJsonSchema;
                    });
                });
            })
                .then(() => result)
                .catch(reason => {
                const errorMsg = `Error fetching API Schema. ${reason === null || reason === void 0 ? void 0 : reason.message}`;
                spinner.fail(errorMsg);
                const result = {
                    success: false,
                    error: errorMsg,
                    targetVersion: null,
                    selectedVersion: null,
                    found: false,
                    foundExact: false,
                    k8sJsonSchema: null
                };
                return result;
            });
        });
    }
    fetchLocal(targetVersion) {
        return __awaiter(this, void 0, void 0, function* () {
            const spinner = (0, spinner_1.spinOperation)('Loading K8s API Schema...');
            const result = {
                success: true,
                targetVersion: targetVersion !== null && targetVersion !== void 0 ? targetVersion : null,
                selectedVersion: null,
                found: false,
                foundExact: true,
                k8sJsonSchema: null,
            };
            const k8sApiRegistry = new k8s_api_schema_registry_1.K8sApiSchemaRegistry(this._logger);
            yield k8sApiRegistry.init();
            this._logger.info("TargetVersion: ", targetVersion);
            if (targetVersion) {
                const searchResult = k8sApiRegistry.findVersion(targetVersion);
                result.selectedVersion = searchResult.result;
                result.foundExact = searchResult.foundExact;
                this._logger.info("searchResult: ", searchResult);
            }
            else {
                result.selectedVersion = k8sApiRegistry.findLatestVersion();
                result.foundExact = true;
            }
            if (result.selectedVersion) {
                this._logger.info("SelectedVersion: %s", result.selectedVersion);
                result.k8sJsonSchema = yield k8sApiRegistry.getVersionSchema(result.selectedVersion);
            }
            result.found = result.k8sJsonSchema ? true : false;
            if (!result.found) {
                result.success = false;
                result.error = "Could not find Kubernetes API schema.";
                result.foundExact = false;
                spinner.fail("Could not find Kubernetes API schema.");
            }
            else {
                spinner.complete("Fetched K8s API schema.");
            }
            return result;
        });
    }
}
exports.K8sApiSchemaFetcher = K8sApiSchemaFetcher;
//# sourceMappingURL=k8s-api-schema-fetcher.js.map