"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.massageLintOptions = exports.command = void 0;
const logger_1 = require("../../logger");
const k8s_api_schema_fetcher_1 = require("../../api-schema/k8s-api-schema-fetcher");
const default_namespace_setter_1 = require("../../processors/default-namespace-setter");
const k8s_cluster_connector_1 = require("../../k8s-connector/k8s-cluster-connector");
const manifests_loader_1 = require("../../manifests/manifests-loader");
const k8s_package_validator_1 = require("../../validation/k8s-package-validator");
const manifest_package_1 = require("../../manifests/manifest-package");
const path_resolver_1 = require("../../path-resolver");
const spinner_1 = require("../../screen/spinner");
const input_source_extractor_1 = require("../../input/input-source-extractor");
const myLogger = logger_1.logger.sublogger('LintCommand');
function command(paths, options) {
    return __awaiter(this, void 0, void 0, function* () {
        logger_1.logger.info("[PATH] ", paths);
        logger_1.logger.info("[OPTIONS] ", options);
        const inputSourceExtractor = new input_source_extractor_1.InputSourceExtractor(logger_1.logger, {
            gitignore: options.gitignore,
            ignorePatterns: options.ignorePatterns
        });
        yield inputSourceExtractor.init();
        {
            const sourceExtractorSpinner = (0, spinner_1.spinOperation)('Identifying manifest sources...');
            {
                const pathResolver = new path_resolver_1.PathResolver();
                inputSourceExtractor.addMany([pathResolver.cliCrdsDir]);
            }
            inputSourceExtractor.addMany(paths);
            yield inputSourceExtractor.extractSources();
            yield inputSourceExtractor.reconcile();
            sourceExtractorSpinner.complete('Sources identified.');
        }
        const manifestPackage = new manifest_package_1.ManifestPackage(logger_1.logger);
        const manifestLoader = new manifests_loader_1.ManifestLoader(logger_1.logger, manifestPackage, inputSourceExtractor, {
            ignoreNonK8s: options.ignoreNonK8s
        });
        yield manifestLoader.loadFromExtractor();
        if (options.stream) {
            yield manifestLoader.loadFromStream();
        }
        manifestPackage.debugOutput();
        let k8sSchemaInfo = null;
        const k8sConnector = new k8s_cluster_connector_1.K8sClusterConnector(logger_1.logger);
        yield k8sConnector.setup(options.liveK8s, options.kubeconfig, options.k8sSkipTlsVerify);
        if (k8sConnector.isUsed) {
            if (!k8sConnector.isConnected) {
                console.log('Could not connect to Kubernetes cluster.');
                process.exit(98);
            }
        }
        const k8sApiSchemaFetcher = new k8s_api_schema_fetcher_1.K8sApiSchemaFetcher(logger_1.logger);
        if (options.k8sVersion || !options.liveK8s) {
            k8sSchemaInfo = yield k8sApiSchemaFetcher.fetchLocal(options.k8sVersion);
        }
        else {
            k8sSchemaInfo = yield k8sApiSchemaFetcher.fetchRemote(k8sConnector);
        }
        if (!k8sSchemaInfo.success) {
            console.log('Could not fetch Kubernetes API Schema. ');
            console.log(k8sSchemaInfo.error);
            process.exit(98);
        }
        logger_1.logger.info("Schema Fetch Success: %s", k8sSchemaInfo.success);
        logger_1.logger.info("TargetK8sVersion: %s", k8sSchemaInfo.targetVersion);
        logger_1.logger.info("TargetK8sVersion: %s", k8sSchemaInfo.targetVersion);
        logger_1.logger.info("SelectedK8sVersion: %s", k8sSchemaInfo.selectedVersion);
        logger_1.logger.info("Found: %s", k8sSchemaInfo.found);
        logger_1.logger.info("FoundExact: %s", k8sSchemaInfo.foundExact);
        if (!k8sSchemaInfo.k8sJsonSchema) {
            console.log('Could not find K8s version: ', k8sSchemaInfo.targetVersion);
            console.log('Select from available versions by running:');
            console.log('$ kubevious list-known-k8s-versions');
            process.exit(99);
        }
        const packageValidator = new k8s_package_validator_1.K8sPackageValidator(logger_1.logger, k8sSchemaInfo.k8sJsonSchema, manifestPackage, {
            ignoreUnknown: options.ignoreUnknown ? true : false,
            skipApplyCrds: options.skipApplyCrds ? true : false,
        });
        yield packageValidator.validate();
        {
            const processor = new default_namespace_setter_1.DefaultNamespaceSetter(logger_1.logger, packageValidator.k8sJsonSchema, manifestPackage);
            processor.process();
        }
        manifestPackage.produceNamespaces();
        manifestPackage.debugOutput();
        return {
            k8sConnector,
            manifestPackage,
            k8sSchemaInfo,
            inputSourceExtractor,
            manifestLoader
        };
    });
}
exports.command = command;
function massageLintOptions(options) {
    var _a, _b, _c, _d, _e, _f, _g;
    options = options !== null && options !== void 0 ? options : {};
    return {
        k8sVersion: options.k8sVersion,
        ignoreUnknown: (_a = options.ignoreUnknown) !== null && _a !== void 0 ? _a : false,
        ignoreNonK8s: (_b = options.ignoreNonK8s) !== null && _b !== void 0 ? _b : false,
        stream: (_c = options.stream) !== null && _c !== void 0 ? _c : false,
        skipApplyCrds: (_d = options.skipApplyCrds) !== null && _d !== void 0 ? _d : false,
        liveK8s: (_e = options.liveK8s) !== null && _e !== void 0 ? _e : false,
        kubeconfig: options.kubeconfig,
        k8sSkipTlsVerify: (_f = options.k8sSkipTlsVerify) !== null && _f !== void 0 ? _f : false,
        gitignore: options.gitignore,
        ignorePatterns: (_g = options.ignorePatterns) !== null && _g !== void 0 ? _g : []
    };
}
exports.massageLintOptions = massageLintOptions;
//# sourceMappingURL=command.js.map