"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ManifestPackage = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const k8s_manifest_1 = require("./k8s-manifest");
const manifest_source_1 = require("./manifest-source");
const base_object_1 = require("../types/base-object");
const result_1 = require("../types/result");
class ManifestPackage extends base_object_1.BaseObject {
    constructor(logger) {
        super();
        this._rootSource = new manifest_source_1.ManifestSource('root', '', null);
        this._manifests = [];
        this._namespaces = [];
        this._logger = logger.sublogger('ManifetPackage');
    }
    get sources() {
        return this._rootSource.childSources;
    }
    get manifests() {
        return this._manifests;
    }
    get namespaces() {
        return this._namespaces;
    }
    getSource(kind, path, originalSource, parentSource) {
        if (!parentSource) {
            parentSource = this._rootSource;
        }
        return parentSource.getSource(kind, path, originalSource);
    }
    addManifest(source, k8sObject) {
        const k8sManifest = new k8s_manifest_1.K8sManifest(k8sObject, source);
        source.addManifest(k8sManifest);
        this._manifests.push(k8sManifest);
        return k8sManifest;
    }
    produceNamespaces() {
        const namespaces = {};
        for (const x of this.manifests) {
            if (x.id.namespace) {
                namespaces[x.id.namespace] = true;
            }
        }
        this._namespaces = the_lodash_1.default.keys(namespaces);
    }
    debugOutput() {
        this._logger.info('[ManifestPackage] BEGIN SOURCES:');
        this._debugOutputSource(this._rootSource, 0);
        this._logger.info('[ManifestPackage] BEGIN MANIFESTS:');
        for (const manifest of this.manifests) {
            this._logger.info("[ManifestPackage] |> %s", manifest.idKey);
            this._debugOutputSourceTree(manifest.source, 1);
        }
        this._logger.info('[ManifestPackage] END');
    }
    _debugOutputSource(source, indent) {
        var _a, _b;
        this._logger.info("[ManifestPackage] %s| + [%s] %s", " ".repeat(indent * 2), source.id.kind.toUpperCase(), source.id.path);
        if (source.originalSource) {
            this._logger.info("[ManifestPackage] %s     ORIG: %s :: %s", " ".repeat(indent * 2), source.originalSource.kind, source.originalSource.path);
        }
        if (the_lodash_1.default.keys((_a = source.originalSource) === null || _a === void 0 ? void 0 : _a.suffixes).length > 0) {
            this._logger.info("[ManifestPackage] %s     suffixes: %s", " ".repeat(indent * 2), JSON.stringify((_b = source.originalSource) === null || _b === void 0 ? void 0 : _b.suffixes));
        }
        if (source.selfErrors.length > 0) {
            this._logger.info("[ManifestPackage] %s     Errors: %s", " ".repeat(indent * 2), source.selfErrors.length);
        }
        if (source.selfWarnings.length > 0) {
            this._logger.info("[ManifestPackage] %s     Warnings: %s", " ".repeat(indent * 2), source.selfWarnings.length);
        }
        for (const child of source.childSources) {
            this._debugOutputSource(child, indent + 1);
        }
    }
    _debugOutputSourceTree(source, indent) {
        this._logger.info("[ManifestPackage] %s| + [%s] %s", " ".repeat(indent * 2), source.id.kind.toUpperCase(), source.id.path);
        if (source.parentSource) {
            this._debugOutputSourceTree(source.parentSource, indent + 1);
        }
    }
    yieldChildren() {
        return the_lodash_1.default.flatten([this._rootSource]);
    }
    exportResult() {
        const rootSourceResult = this._rootSource.exportResult();
        const manifestPackageR = {
            severity: 'pass',
            manifests: [],
            rootSource: rootSourceResult,
        };
        for (const manifest of this._manifests) {
            const manifestR = manifest.exportResult();
            manifestPackageR.manifests.push(manifestR);
        }
        manifestPackageR.severity = (0, result_1.makeObjectSeverityFromChildren)(manifestPackageR.severity, manifestPackageR.manifests);
        manifestPackageR.severity = (0, result_1.makeObjectSeverityFromChildren)(manifestPackageR.severity, [rootSourceResult]);
        return manifestPackageR;
    }
}
exports.ManifestPackage = ManifestPackage;
//# sourceMappingURL=manifest-package.js.map