"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RuleExecutionRuntime = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const the_promise_1 = require("the-promise");
const manifest_violation_1 = require("./manifest-violation");
class RuleExecutionRuntime {
    constructor(logger, manifest, ruleObject, ruleRuntime, application, values) {
        this._logger = logger.sublogger('RuleExecutionRuntime');
        this._manifest = manifest;
        this._ruleObject = ruleObject;
        this._ruleRuntime = ruleRuntime;
        this._application = application;
        this._values = values;
    }
    get compiler() {
        return this._ruleRuntime.compiler;
    }
    get rule() {
        return this._ruleRuntime.rule;
    }
    get application() {
        return this._application;
    }
    get isCompiled() {
        return this._ruleRuntime.isCompiled;
    }
    get hasRuntimeErrors() {
        return this._ruleRuntime.hasRuntimeErrors;
    }
    get ruleEngineReporter() {
        return this._ruleRuntime.ruleEngineReporter;
    }
    execute() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.info('[execute] %s :: %s :: %s...', this._ruleObject.kind, this._ruleObject.namespace, this._ruleObject.name);
            if (!this.compiler) {
                return;
            }
            if (this.compiler.isCompiled) {
                const results = yield this._processTarget();
                this._logger.info('[execute] Targets Count: %s', results.length);
                const cacheResult = yield this._ruleRuntime.processGlobalCache(this._values);
                if (!cacheResult.success) {
                    return;
                }
                yield the_promise_1.MyPromise.serial(results, x => this._processItem(x, cacheResult));
            }
            if (!this.isCompiled || this.hasRuntimeErrors) {
                if (!((_a = this._manifest.errorsWithRule) !== null && _a !== void 0 ? _a : false)) {
                    this._manifest.errorsWithRule = true;
                    this._manifest.reportError("Failed to compile the rule.");
                }
            }
        });
    }
    _processTarget() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.compiler.targetProcessor.execute(this._application, this._values);
            }
            catch (reason) {
                this._logger.error("Failed to execute the rule %s", this._ruleObject.name, reason);
                this.compiler.reportScriptErrors('rule', [reason.message]);
                return [];
            }
        });
    }
    _processItem(item, globalCacheResult) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.info("[_processItem] %s :: %s", item.apiVersion, item.kind);
            const cacheResult = yield this._ruleRuntime.processLocalCache(item, this._values);
            if (!cacheResult.success) {
                return;
            }
            return this._processValidation(item, globalCacheResult, cacheResult);
        });
    }
    _processValidation(item, globalCacheResult, localCacheResult) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.info("[_processValidation] %s :: %s", item.apiVersion, item.kind);
            const result = yield this.compiler.validationProcessor.execute(item, globalCacheResult.cache, localCacheResult.cache, this._values);
            this._logger.debug("[_processValidation]  result: ", result);
            if (result.success) {
                this._processValidationResult(item.manifest, result);
            }
            else {
                if (result.messages && (result.messages.length > 0)) {
                    this.compiler.reportScriptErrors('rule', result.messages);
                }
                else {
                    this.compiler.reportScriptErrors('rule', ['Unknown error executing the rule.']);
                }
            }
        });
    }
    _processValidationResult(manifest, result) {
        manifest.rules.processed = true;
        if ((the_lodash_1.default.keys(result.validation.errorMsgs).length + the_lodash_1.default.keys(result.validation.warnMsgs).length) > 0) {
            if (the_lodash_1.default.keys(result.validation.errorMsgs).length > 0) {
                manifest.rules.errors = true;
            }
            if (the_lodash_1.default.keys(result.validation.warnMsgs).length > 0) {
                manifest.rules.warnings = true;
            }
            const manifestViolations = new manifest_violation_1.ManifestViolation(manifest);
            for (const error of the_lodash_1.default.keys(result.validation.errorMsgs)) {
                this._reportError(manifest, error);
                manifestViolations.reportError(error);
            }
            for (const warn of the_lodash_1.default.keys(result.validation.warnMsgs)) {
                this._reportWarning(manifest, warn);
                manifestViolations.reportWarning(warn);
            }
            this._ruleRuntime.violations.push(manifestViolations);
        }
        else {
            this._ruleRuntime.passed.push(manifest);
        }
    }
    _reportError(manifest, msg) {
        const manifestMsg = `Rule ${this._ruleObject.name} violated. ${msg}.`;
        this.ruleEngineReporter.reportError(this._ruleObject, manifest, manifestMsg);
    }
    _reportWarning(manifest, msg) {
        const manifestMsg = `Rule ${this._ruleObject.name} warned. ${msg}.`;
        this.ruleEngineReporter.reportWarning(this._ruleObject, manifest, manifestMsg);
    }
}
exports.RuleExecutionRuntime = RuleExecutionRuntime;
//# sourceMappingURL=rule-execution-runtime.js.map