"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeValues = exports.RuleRuntime = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const the_promise_1 = require("the-promise");
const types_1 = require("../registry/types");
const rule_compiler_1 = require("../compiler/rule-compiler");
const rule_execution_runtime_1 = require("./rule-execution-runtime");
const result_1 = require("../../types/result");
class RuleRuntime {
    constructor(logger, rule, rulesRuntime) {
        this._executors = [];
        this._compiler = undefined;
        this._violations = [];
        this._passed = [];
        this._isSkipped = false;
        this._ruleCacheData = {
            global: null,
            cluster: null,
            namespaces: {}
        };
        this._logger = logger.sublogger('RuleRuntime');
        this._rule = rule;
        this._rulesRuntime = rulesRuntime;
    }
    get rule() {
        return this._rule;
    }
    get ruleObject() {
        return this._rule;
    }
    get manifest() {
        return this._rule.manifest;
    }
    get compiler() {
        return this._compiler;
    }
    get isCompiled() {
        var _a, _b;
        return (_b = (_a = this._compiler) === null || _a === void 0 ? void 0 : _a.isCompiled) !== null && _b !== void 0 ? _b : false;
    }
    get hasRuntimeErrors() {
        var _a, _b;
        return (_b = (_a = this._compiler) === null || _a === void 0 ? void 0 : _a.hasRuntimeErrors) !== null && _b !== void 0 ? _b : false;
    }
    get ruleErrors() {
        var _a, _b;
        return (_b = (_a = this._compiler) === null || _a === void 0 ? void 0 : _a.ruleErrors) !== null && _b !== void 0 ? _b : [];
    }
    get violations() {
        return this._violations;
    }
    get passed() {
        return this._passed;
    }
    get ruleEngineReporter() {
        return this._rulesRuntime.ruleEngineReporter;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._rule.hasUnmedDependency) {
                this._logger.info("[init] skipping %s. Has Unmet Dependency.", this.rule.name);
            }
            if (this._rule.isDisabled) {
                this._logger.info("[init] skipping %s. Is Disabled.", this.rule.name);
                this._isSkipped = true;
                return;
            }
            if (this._rulesRuntime.isRuleSkipped(this._rule)) {
                this._logger.info("[init] skipping %s. Is Skipped.", this.rule.name);
                this._isSkipped = true;
                return;
            }
            yield this._prepareRule();
        });
    }
    _prepareRule() {
        return __awaiter(this, void 0, void 0, function* () {
            // this._logger.info("[_prepareRule] %s :: %s ::%s...", ruleObject.kind, ruleObject.namespace, ruleObject.name)
            const compiler = new rule_compiler_1.RuleCompiler(this._logger, this.ruleObject, this._rulesRuntime.targetExecutionContext, this._rulesRuntime.validatorExecutionContext);
            yield compiler.compile();
            this._logger.info("[_prepareRule]   - isCompiled: %s", compiler.isCompiled);
            this._logger.info("[_prepareRule]   - hasRuntimeErrors: %s", compiler.hasRuntimeErrors);
            this._logger.info("[_prepareRule]   - errors: ", compiler.ruleErrors);
            for (const error of compiler.ruleErrors) {
                this.manifest.reportError(error.msg);
            }
            this._compiler = compiler;
        });
    }
    setupApplication(application, values) {
        if (!this._compiler) {
            return;
        }
        application = application !== null && application !== void 0 ? application : {};
        values = values !== null && values !== void 0 ? values : {};
        values = makeValues([values, this._rule.values]);
        const ruleExecutionRuntime = new rule_execution_runtime_1.RuleExecutionRuntime(this._logger, this.manifest, this._rule, this, application, values);
        this._executors.push(ruleExecutionRuntime);
    }
    execute() {
        this._logger.info("[execute] Rule: %s. Executors: %s", this.rule.name, this._executors.length);
        return the_promise_1.MyPromise.serial(this._executors, x => this._executeRule(x));
    }
    _executeRule(executor) {
        this._logger.info("[_executeRule] applicator: ", executor.application);
        return the_promise_1.MyPromise.timeout(1)
            .then(() => executor.execute());
    }
    exportResult() {
        var _a;
        const ruleManifestResult = this.manifest.exportResult();
        const ruleErrors = ((_a = this.ruleErrors) !== null && _a !== void 0 ? _a : []).map(x => x.msg);
        for (const error of ruleErrors) {
            if (!ruleManifestResult.messages) {
                ruleManifestResult.messages = [];
            }
            ruleManifestResult.messages.push({ severity: 'error', msg: error });
        }
        // TODO: Take this out later and avoid duplicates in the first place
        ruleManifestResult.messages = the_lodash_1.default.uniqBy(ruleManifestResult.messages, x => the_lodash_1.default.stableStringify(x));
        (0, result_1.setupBaseObjectSeverity)(ruleManifestResult);
        const violations = this._violations.map(x => x.exportResult());
        const hasViolationErrors = the_lodash_1.default.some(violations, x => x.severity === 'fail');
        const hasViolationWarnings = the_lodash_1.default.some(violations, x => x.severity === 'warning');
        let ruleSeverity = 'pass';
        if (hasViolationErrors) {
            ruleSeverity = 'fail';
        }
        else if (hasViolationWarnings) {
            ruleSeverity = 'warning';
        }
        const result = {
            ruleManifest: ruleManifestResult,
            ruleCategories: this._rule.categories,
            isSkipped: this._isSkipped,
            compiled: this.isCompiled && !this.hasRuntimeErrors,
            pass: !hasViolationErrors,
            ruleSeverity: ruleSeverity,
            hasViolationErrors: hasViolationErrors,
            hasViolationWarnings: hasViolationWarnings,
            violations: violations,
            passed: this.passed.map(x => this._exportPassedManifest(x))
        };
        if (this.rule.kind == types_1.RuleKind.Rule) {
            result.namespace = this.rule.namespace;
        }
        return result;
    }
    processGlobalCache(values) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._compiler || !this._compiler.isCompiled) {
                return {
                    success: false,
                    cache: {}
                };
            }
            if (!this._compiler.globalCacheProcessor) {
                return {
                    success: true,
                    cache: {}
                };
            }
            // this._logger.info("[processGlobalCache] %s :: %s", item.apiVersion, item.kind);
            if (this._ruleCacheData.global) {
                return {
                    success: true,
                    cache: this._ruleCacheData.global.cache
                };
            }
            const result = yield this._compiler.globalCacheProcessor.execute(null, values);
            const clusterData = {
                cache: result.cache
            };
            this._ruleCacheData.global = clusterData;
            return {
                success: true,
                cache: clusterData.cache
            };
        });
    }
    processLocalCache(item, values) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._compiler || !this._compiler.isCompiled) {
                return {
                    success: false,
                    cache: {}
                };
            }
            if (!this._compiler.localCacheProcessor) {
                return {
                    success: true,
                    cache: {}
                };
            }
            const namespace = item.namespace;
            if (namespace) {
                if (this._ruleCacheData.namespaces[namespace]) {
                    return {
                        success: true,
                        cache: this._ruleCacheData.namespaces[namespace].cache
                    };
                }
                const result = yield this._compiler.localCacheProcessor.execute(namespace, values);
                const namespaceData = {
                    cache: result.cache
                };
                this._ruleCacheData.namespaces[namespace] = namespaceData;
                return {
                    success: true,
                    cache: namespaceData.cache
                };
            }
            else {
                if (this._ruleCacheData.cluster) {
                    return {
                        success: true,
                        cache: this._ruleCacheData.cluster.cache
                    };
                }
                const result = yield this._compiler.localCacheProcessor.execute(null, values);
                const clusterData = {
                    cache: result.cache
                };
                this._ruleCacheData.cluster = clusterData;
                return {
                    success: true,
                    cache: clusterData.cache
                };
            }
        });
    }
    _exportPassedManifest(manifest) {
        const result = Object.assign(Object.assign({}, manifest.id), { severity: 'pass', sources: manifest.exportSourcesResult() });
        return result;
    }
}
exports.RuleRuntime = RuleRuntime;
function makeValues(valuesArray) {
    valuesArray = valuesArray.map(x => x);
    let result = {};
    for (const x of valuesArray) {
        result = the_lodash_1.default.defaults(result, x);
    }
    return result;
}
exports.makeValues = makeValues;
//# sourceMappingURL=rule-runtime.js.map