"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RulesRuntime = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const the_promise_1 = require("the-promise");
const execution_context_1 = require("./execution-context");
const rule_engine_reporter_1 = require("../reporting/rule-engine-reporter");
const spinner_1 = require("../../screen/spinner");
const version_checker_1 = require("../../utils/version-checker");
const result_1 = require("../../types/result");
const rule_runtime_1 = require("./rule-runtime");
class RulesRuntime {
    constructor(logger, ruleRegistry, manifestPackage, namespaces, targetQueryExecutor, validatorQueryExecutor, skipOptions) {
        this._rules = [];
        this._clusterRulesDict = {};
        this._logger = logger.sublogger('RulesRuntime');
        this._ruleRegistry = ruleRegistry;
        this._namespaces = namespaces;
        this._manifestPackage = manifestPackage;
        this._targetExecutionContext = new execution_context_1.ExecutionContext(this._logger, targetQueryExecutor, manifestPackage);
        this._validatorExecutionContext = new execution_context_1.ExecutionContext(this._logger, validatorQueryExecutor, manifestPackage);
        this._ruleEngineReporter = new rule_engine_reporter_1.RuleEngineReporter(this._logger, manifestPackage);
        this._skipRulesDict = the_lodash_1.default.makeBoolDict(skipOptions.skipRules);
        this._onlyRulesDict = the_lodash_1.default.makeBoolDict(skipOptions.onlyRules);
        this._skipRuleCategoriesDict = the_lodash_1.default.makeBoolDict(skipOptions.skipRuleCategories);
        this._onlyRuleCategoriesDict = the_lodash_1.default.makeBoolDict(skipOptions.onlyRuleCategories);
    }
    get rules() {
        return this._rules;
    }
    get targetExecutionContext() {
        return this._targetExecutionContext;
    }
    get validatorExecutionContext() {
        return this._validatorExecutionContext;
    }
    get ruleEngineReporter() {
        return this._ruleEngineReporter;
    }
    init() {
        return Promise.resolve()
            .then(() => this._checkDependencies())
            .then(() => this._initClusterRules())
            .then(() => this._initNamespaceRules())
            .then(() => this._initRuleApplicators());
    }
    execute() {
        this._logger.info("[execute] Begin. Count: %s", this._rules.length);
        this._spinner = (0, spinner_1.spinOperation)('Validating rules...');
        return the_promise_1.MyPromise.serial(this._rules, x => this._executeRule(x))
            .then(() => {
            this._spinner.complete('Rules validation complete.');
        });
    }
    exportResult() {
        const result = {
            success: true,
            severity: 'pass',
            rules: this.rules.map(x => x.exportResult()),
        };
        // result.severity = makeObjectSeverityFromChildren(result.severity, result.rules.map(x => x.ruleManifest));
        result.severity = (0, result_1.makeObjectSeverity)(result.severity, result.rules.map(x => x.ruleSeverity));
        result.success = result.severity == 'pass' || result.severity == 'warning';
        return result;
    }
    isRuleSkipped(rule) {
        if (this._skipRulesDict[rule.name]) {
            return true;
        }
        if (the_lodash_1.default.keys(this._onlyRulesDict).length > 0) {
            if (!this._onlyRulesDict[rule.name]) {
                return true;
            }
        }
        for (const category of rule.categories) {
            if (this._skipRuleCategoriesDict[category]) {
                return true;
            }
        }
        for (const category of the_lodash_1.default.keys(this._onlyRuleCategoriesDict)) {
            if (!rule.categoryDict[category]) {
                return true;
            }
        }
        return false;
    }
    _checkDependencies() {
        for (const rule of this._ruleRegistry.clusterRules) {
            this._checkRuleDependencies(rule);
        }
        for (const rule of this._ruleRegistry.rules) {
            this._checkRuleDependencies(rule);
        }
    }
    _checkRuleDependencies(rule) {
        for (const dependency of rule.dependencies) {
            if (rule.isDisabled) {
                return;
            }
            this._checkRuleDependency(rule, dependency);
        }
    }
    _checkRuleDependency(rule, dependency) {
        if (dependency.name === 'kubevious') {
            const issues = (0, version_checker_1.checkKubeviousVersion)(dependency.minVersion, dependency.maxVersion, dependency.range);
            if (issues.length > 0) {
                if (!rule.hasUnmedDependency) {
                    rule.manifest.reportWarning(`Has unsupported Kubevious CLI dependency. ${issues.join(". ")}`);
                    rule.hasUnmedDependency = true;
                    rule.isDisabled = true;
                }
            }
        }
    }
    _initClusterRules() {
        return the_promise_1.MyPromise.serial(this._ruleRegistry.clusterRules, x => {
            return this._initClusterRule(x);
        });
    }
    _initClusterRule(clusterRule) {
        return __awaiter(this, void 0, void 0, function* () {
            const r = new rule_runtime_1.RuleRuntime(this._logger, clusterRule, this);
            this._clusterRulesDict[r.rule.name] = r;
            this._rules.push(r);
            yield r.init();
            this._logger.info("[_initClusterRule] clustered: %s", clusterRule.clustered);
            this._logger.info("[_initClusterRule] useApplicator: %s", clusterRule.useApplicator);
            if (clusterRule.clustered) {
                const application = {};
                r.setupApplication(application, clusterRule.values);
            }
            else {
                if (clusterRule.useApplicator) {
                    return;
                }
                let namespaces = this._namespaces;
                if (clusterRule.onlySelectedNamespaces) {
                    namespaces = namespaces.filter(x => clusterRule.namespaces[x]);
                }
                for (const namespace of namespaces) {
                    const application = {
                        namespace: namespace
                    };
                    const ruleNamespaceInfo = clusterRule.namespaces[namespace];
                    r.setupApplication(application, ruleNamespaceInfo === null || ruleNamespaceInfo === void 0 ? void 0 : ruleNamespaceInfo.values);
                }
            }
        });
    }
    _initNamespaceRules() {
        return the_promise_1.MyPromise.serial(this._ruleRegistry.rules, x => {
            return this._initNamespaceRule(x);
        });
    }
    _initNamespaceRule(rule) {
        this._logger.info("[_initNamespaceRule] %s :: %s...", rule.namespace, rule.name);
        const r = new rule_runtime_1.RuleRuntime(this._logger, rule, this);
        this._rules.push(r);
        const application = {
            namespace: rule.namespace
        };
        r.setupApplication(application);
    }
    _initRuleApplicators() {
        return the_promise_1.MyPromise.serial(this._ruleRegistry.ruleApplicators, x => {
            return this._initRuleApplicator(x);
        });
    }
    _initRuleApplicator(applicator) {
        this._logger.info("[_initRuleApplicator] %s :: %s...", applicator.namespace, applicator.name);
        if (applicator.spec.disabled) {
            this._logger.info("[_initRuleApplicator] Disabled: %s :: %s", applicator.namespace, applicator.name);
            return;
        }
        const clusterRefName = applicator.spec.clusterRuleRef.name;
        if (!clusterRefName) {
            this._logger.info("[_initRuleApplicator] No clusterRefName: %s :: %s", applicator.namespace, applicator.name);
            return;
        }
        const clusterRuleRuntime = this._clusterRulesDict[clusterRefName];
        if (!clusterRuleRuntime) {
            this._logger.info("[_initRuleApplicator] Missing Cluster Rule: %s", clusterRefName);
            applicator.manifest.reportError(`ClusterRule ${clusterRefName} not found`);
            return;
        }
        const clusterRule = clusterRuleRuntime.rule;
        if (!clusterRule.useApplicator) {
            this._logger.info("[_initRuleApplicator] Cluster Rule not using applicators: %s", clusterRefName);
            applicator.manifest.reportError(`ClusterRule ${clusterRefName} not using applicators`);
            return;
        }
        if (clusterRule.isDisabled) {
            this._logger.info("[_initRuleApplicator] Cluster Rule disabled: %s", clusterRefName);
            return;
        }
        if (clusterRuleRuntime.compiler) {
            if (!clusterRuleRuntime.compiler.isCompiled) {
                this._logger.info("[_initRuleApplicator] Cluster Rule has compilation errors: %s", clusterRefName);
                applicator.manifest.reportError(`ClusterRule ${clusterRefName} has compilation errors`);
                return;
            }
        }
        const ruleNamespaceInfo = clusterRule.namespaces[applicator.namespace];
        if (clusterRule.onlySelectedNamespaces) {
            if (!ruleNamespaceInfo) {
                return;
            }
        }
        const application = {
            namespace: applicator.namespace
        };
        const values = (0, rule_runtime_1.makeValues)([applicator.values, ruleNamespaceInfo === null || ruleNamespaceInfo === void 0 ? void 0 : ruleNamespaceInfo.values]);
        clusterRuleRuntime.setupApplication(application, values);
    }
    _executeRule(rule) {
        var _a;
        this._logger.info("[_executeRule] %s...", rule.rule.name);
        (_a = this._spinner) === null || _a === void 0 ? void 0 : _a.update(`Validating rule: ${rule.rule.name}...`);
        return the_promise_1.MyPromise.timeout(1)
            .then(() => rule.execute());
    }
}
exports.RulesRuntime = RulesRuntime;
//# sourceMappingURL=rules-runtime.js.map