"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.K8sManifestValidator = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const ajv_1 = __importDefault(require("ajv"));
const ajv_formats_1 = __importDefault(require("ajv-formats"));
const k8s_1 = require("../utils/k8s");
class K8sManifestValidator {
    constructor(logger, k8sJsonSchema, params) {
        this._logger = logger.sublogger('K8sManifestValidator');
        this._k8sJsonSchema = k8sJsonSchema;
        this._params = params || {};
    }
    validate(k8sManifest) {
        var _a, _b;
        try {
            const apiResource = (0, k8s_1.getApiResourceId)(k8sManifest);
            if (!apiResource) {
                return {
                    success: false,
                    errors: ["Invalid manifest. Make sure that apiVersion and kind are set."]
                };
            }
            const resourceKey = (0, k8s_1.getJsonSchemaResourceKey)(apiResource);
            this._logger.verbose('[validate] resourceKey: ', resourceKey);
            const resourceInfo = this._k8sJsonSchema.resources[resourceKey];
            this._logger.verbose('[validate] resourceInfo: ', resourceInfo);
            if (!resourceInfo) {
                const msg = `Unknown API Resource. apiVersion: ${k8sManifest.apiVersion}, kind: ${k8sManifest.kind}.`;
                if (this._params.ignoreUnknown) {
                    return {
                        success: true,
                        warnings: [msg]
                    };
                }
                return {
                    success: false,
                    errors: [msg]
                };
            }
            const schema = {
                ['$ref']: `#/definitions/${resourceInfo.definitionId}`,
                definitions: this._k8sJsonSchema.definitions
            };
            this._logger.verbose('[validate] %s', resourceInfo.definitionId);
            this._preProcessNode(k8sManifest, {
                ['$ref']: schema['$ref']
            }, null, null);
            // this._logger.info("PreProcessed Manifest: ", k8sManifest);
            const ajvOptions = {
                strict: true,
                discriminator: true,
                strictSchema: true,
                validateFormats: true,
                strictRequired: true,
                strictTypes: true,
                strictNumbers: true,
                formats: {}
            };
            const ajv = new ajv_1.default(ajvOptions);
            (0, ajv_formats_1.default)(ajv);
            const validator = ajv.compile(schema);
            const result = validator(k8sManifest);
            // this._logger.info("RESULT: ", result);
            // this._logger.info("ERRORS: ", validator.errors);
            if (result) {
                return {
                    success: true
                };
            }
            else {
                return {
                    success: false,
                    errors: ((_a = validator.errors) !== null && _a !== void 0 ? _a : []).map(x => this._mapError(x))
                };
            }
        }
        catch (reason) {
            this._logger.info("[validate] Catch: ", reason);
            return {
                success: false,
                errors: [(_b = reason === null || reason === void 0 ? void 0 : reason.message) !== null && _b !== void 0 ? _b : 'Unkown Error.']
            };
        }
    }
    _preProcessNode(node, schema, parentNode, parentProp) {
        // this._logger.info('[_preProcessNode] ')
        if (!schema) {
            return;
        }
        {
            const refName = schema['$ref'];
            if (refName) {
                return this._preProcessNodeRef(node, refName, parentNode, parentProp);
            }
        }
        {
            const allOf = schema.allOf;
            if (allOf) {
                const firstSchema = the_lodash_1.default.first(allOf);
                return this._preProcessNode(node, firstSchema, parentNode, parentProp);
            }
        }
        const sType = schema.type;
        // this._logger.info('[_preProcessNode] type: %s', sType)
        if (sType === 'object') {
            if (the_lodash_1.default.isNull(node)) {
                // this._logger.error(">>>>>> OBJECT IS NULL");
                if (parentNode) {
                    parentNode[parentProp] = {};
                }
            }
            else if (the_lodash_1.default.isObject(node)) {
                if (schema.properties) {
                    this._preProcessClearEmptyRequiredStrings(node, schema);
                    for (const propName of the_lodash_1.default.keys(node)) {
                        // this._logger.info('[_preProcessNode] propName: %s', propName)
                        const propNode = node[propName];
                        const propSchema = schema.properties[propName];
                        if (propSchema) {
                            this._preProcessNode(propNode, propSchema, node, propName);
                        }
                        // this._logger.info('[_preProcessNode] propNode: ', propNode)
                    }
                }
            }
        }
        else if (sType === 'array') {
            const itemsSchema = schema.items;
            if (the_lodash_1.default.isNull(node)) {
                // this._logger.error(">>>>>> ARRAY IS NULL");
                if (parentNode) {
                    parentNode[parentProp] = [];
                }
            }
            else if (the_lodash_1.default.isArray(node)) {
                for (const childNode of node) {
                    this._preProcessNode(childNode, itemsSchema, null, null);
                }
            }
        }
    }
    _preProcessClearEmptyRequiredStrings(node, schema) {
        if (schema.required) {
            for (const propName of schema.required) {
                const propSchema = schema.properties[propName];
                if (propSchema &&
                    propSchema.type == "string") {
                    const propNode = node[propName];
                    if (the_lodash_1.default.isString(propNode) && (propNode.length === 0)) {
                        if (!the_lodash_1.default.isNullOrUndefined(propSchema.default)) {
                            node[propName] = propSchema.default;
                        }
                        // Before we used to delete the node. This behavior causes
                        // a false possitive when an empty required string fails validation.
                        // https://github.com/kubevious/cli/issues/20
                        // In future should try to set the value of undefined values as default value
                        // of the schema. That would also help with guard functionality with validation
                        // that depends on default values.
                        // Old Code: delete (node as any)[propName];
                    }
                }
            }
        }
    }
    _preProcessNodeRef(node, refName, parentNode, parentProp) {
        // this._logger.info('[_preProcessNodeRef] refName: %s', refName);
        const prefix = '#/definitions/';
        if (the_lodash_1.default.startsWith(refName, prefix)) {
            const key = refName.substring(prefix.length);
            // this._logger.info('[_preProcessNodeRef] key: %s', key);
            const refSchema = this._k8sJsonSchema.definitions[key];
            // this._logger.info('[_preProcessNodeRef] refSchema: ', refSchema);
            return this._preProcessNode(node, refSchema, parentNode, parentProp);
        }
    }
    _mapError(error) {
        var _a, _b, _c;
        if (error.keyword === 'type') {
            return `Invalid type under "${error.instancePath}". ${the_lodash_1.default.upperFirst(error.message)}.`;
        }
        if (error.keyword === 'additionalProperties') {
            if (error.instancePath) {
                return `Unknown property "${error.params.additionalProperty}" under "${error.instancePath}"`;
            }
            return `Unknown property "${error.params.additionalProperty}"`;
        }
        if (error.keyword === 'required') {
            if (error.instancePath) {
                return `Required property "${error.params.missingProperty}" missing under "${error.instancePath}"`;
            }
            return `Required property "${error.params.missingProperty}" missing.`;
        }
        if (error.keyword === 'enum') {
            const allowedValues = (_b = (_a = error.params) === null || _a === void 0 ? void 0 : _a.allowedValues) !== null && _b !== void 0 ? _b : [];
            return `Unknown enum value provided in "${error.instancePath}". Allowed values are: ${allowedValues.join(', ')}.`;
        }
        return (_c = error.message) !== null && _c !== void 0 ? _c : 'unknown error';
    }
}
exports.K8sManifestValidator = K8sManifestValidator;
//# sourceMappingURL=k8s-manifest-validator.js.map