"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.K8sPackageValidator = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const the_promise_1 = require("the-promise");
const k8s_manifest_validator_1 = require("./k8s-manifest-validator");
const spinner_1 = require("../screen/spinner");
const k8s_1 = require("../utils/k8s");
const crd_schema_to_json_schema_converter_1 = require("../api-schema/crd-schema-to-json-schema-converter");
class K8sPackageValidator {
    constructor(logger, k8sJsonSchema, manifestPackage, params) {
        this._logger = logger.sublogger('K8sPackageValidator');
        this._origK8sJsonSchema = k8sJsonSchema;
        this._manifestPackage = manifestPackage;
        this._params = params || {};
        this._k8sJsonSchema = this._makeSchemaClone();
        // this._postProcessSchemas();
    }
    get k8sJsonSchema() {
        return this._k8sJsonSchema;
    }
    validate() {
        this._spinner = (0, spinner_1.spinOperation)('Linting manifests...');
        return Promise.resolve()
            .then(() => {
            if (!this._params.skipApplyCrds) {
                this._logger.info("[validate] Running CRD Application...");
                return this._applyCRDs();
            }
        })
            .then(() => {
            this._logger.info("[validate] Running Manifest Validation...");
            return the_promise_1.MyPromise.serial(this._manifestPackage.manifests, x => this._validateManifest(x))
                .then(() => {
                this._spinner.complete('Lint complete.');
            });
        });
    }
    _applyCRDs() {
        const crds = this._manifestPackage.manifests.filter(x => (0, k8s_1.isCRD)(x.id));
        return the_promise_1.MyPromise.serial(crds, x => this._applyCRD(x));
    }
    _applyCRD(crdManifest) {
        this._logger.verbose('[_applyCRD] ', crdManifest.id);
        this._fixCRD(crdManifest);
        if (this._isValidCRD(crdManifest)) {
            this._applyCRDToSchema(crdManifest, this._k8sJsonSchema);
        }
    }
    _fixCRD(crdManifest) {
        var _a, _b, _c, _d;
        if (the_lodash_1.default.isNull((_b = (_a = crdManifest.config) === null || _a === void 0 ? void 0 : _a.metadata) === null || _b === void 0 ? void 0 : _b.creationTimestamp)) {
            (_d = (_c = crdManifest.config) === null || _c === void 0 ? void 0 : _c.metadata) === null || _d === void 0 ? true : delete _d.creationTimestamp;
        }
    }
    _applyCRDToSchema(crdManifest, jsonSchema) {
        var _a, _b;
        this._logger.silly('[_applyCRDToSchema] ', crdManifest.id);
        const crd = crdManifest.config;
        const isNamespaced = (crd.spec.scope === 'Namespaced');
        for (const versionSpec of ((_a = crd.spec.versions) !== null && _a !== void 0 ? _a : [])) {
            if (versionSpec.served) {
                const spec = (_b = versionSpec.schema) === null || _b === void 0 ? void 0 : _b.openAPIV3Schema;
                if (spec) {
                    this._applyCRDVersion(crd.spec.group, versionSpec.name, crd.spec.names.kind, isNamespaced, spec, jsonSchema);
                }
            }
        }
    }
    _isValidCRD(crdManifest) {
        this._validateManifest(crdManifest);
        return crdManifest.success;
    }
    _applyCRDVersion(group, version, kind, isNamespaced, crdSchema, jsonSchema) {
        const apiResource = {
            group: group,
            kind: kind,
            version: version,
        };
        this._logger.silly('[_applyCRDToSchema] apiResource: ', apiResource);
        let parts = group.split('.');
        parts = the_lodash_1.default.reverse(parts);
        parts.push(version);
        parts.push(kind);
        const definitionKey = parts.join('.');
        this._logger.silly('[_applyCRDToSchema] definitionKey: %s', definitionKey);
        const resourceKey = (0, k8s_1.getJsonSchemaResourceKey)(apiResource);
        this._logger.silly('[_applyCRDToSchema] apiResource: ', apiResource);
        this._logger.silly('[_applyCRDToSchema] resourceKey: %s', resourceKey);
        jsonSchema.definitions[definitionKey] = this._convertCrdSchemaToJsonSchema(crdSchema);
        jsonSchema.resources[resourceKey] = {
            definitionId: definitionKey,
            namespaced: isNamespaced
        };
    }
    _convertCrdSchemaToJsonSchema(crdSchema) {
        const converter = new crd_schema_to_json_schema_converter_1.CrdSchemaToJsonSchemaConverter(this._logger);
        return converter.convert(crdSchema);
    }
    _validateManifest(manifest) {
        if (!manifest.source.success) {
            return;
        }
        if (manifest.isLinted) {
            return;
        }
        manifest.isLinted = true;
        this._spinner.update(`Validating ${manifest.source.id.path}...`);
        {
            const jsonSchema = (0, k8s_1.isCRD)(manifest.id) ? this._origK8sJsonSchema : this._k8sJsonSchema;
            const validator = new k8s_manifest_validator_1.K8sManifestValidator(this._logger, jsonSchema, this._params);
            const result = validator.validate(manifest.config);
            if (!result.success) {
                this._logger.verbose("ERRORS: ", result.errors);
                manifest.reportErrors(result.errors);
            }
            manifest.reportWarnings(result.warnings);
        }
        if ((0, k8s_1.isCRD)(manifest.id)) {
            const newJsonSchema = this._makeSchemaClone();
            this._applyCRDToSchema(manifest, newJsonSchema);
            const validator = new k8s_manifest_validator_1.K8sManifestValidator(this._logger, newJsonSchema, this._params);
            const result = validator.validate(manifest.config);
            // this._logger.info("[_validateManifest] isCRD success: %s", result.success);
            if (!result.success) {
                this._logger.verbose("[_validateManifest] isCRD ERRORS: ", result.errors);
                manifest.reportErrors(result.errors);
            }
            manifest.reportWarnings(result.warnings);
        }
    }
    _makeSchemaClone() {
        const newSchema = {
            resources: the_lodash_1.default.clone(this._origK8sJsonSchema.resources),
            definitions: the_lodash_1.default.clone(this._origK8sJsonSchema.definitions)
        };
        return newSchema;
    }
}
exports.K8sPackageValidator = K8sPackageValidator;
//# sourceMappingURL=k8s-package-validator.js.map