"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupPOD_STATUS_NOT_SCHEDULED = void 0;
var validator_id_1 = require("../../../validator-id");
var categories_1 = require("../../categories");
function setupPOD_STATUS_NOT_SCHEDULED(builder) {
    builder
        .validator(validator_id_1.ValidatorID.POD_STATUS_NOT_SCHEDULED)
        .category(categories_1.ValidatorCategory.Compute)
        .subCategory(categories_1.ValidatorSubCategory.Status)
        .title('Stage 1. Pod not scheduled')
        .description("During the launch, Pods go through a sequence of execution stages.\nThe first stage is to assign a Node on which the Pod should launch.\nThis validator detects a condition when a Pod is not yet scheduled to run on a Node.\n\nThere are numerous reasons why Pods can be stuck in the scheduling stage.\nThe reasons can be combinations of insufficient cluster resources,\nsuboptimal node selectors, node affinity, pod affinity and anti-affinity, PodOverhead,\nnode taints, and tolerations. There is also a NodeRestriction admission\ncontroller available as a plugin which can restrict scheduling of certain\nworkloads on certain nodes.\n\nRefer to the links below for details on troubleshooting Pod scheduling failures.\n")
        .affected('Pod')
        .example("\nkind: Pod\n...\nstatus:\n  conditions:\n    - type: PodScheduled\n      status: \"False\"\n      lastProbeTime: null\n      lastTransitionTime: 2022-01-01T00:00:00Z\n")
        .resolution('Check if cluster has enough resources')
        .resolutionAndExample('Check the nodeSelector', "\n# Node Selector\napiVersion: v1\nkind: Pod\nmetadata:\n  name: nginx\nspec:\n  containers:\n  - name: nginx\n    image: nginx\n  nodeSelector:\n    disktype: ssd \n")
        .resolutionAndExample('Check node affinity', "\n# Node Affinity\napiVersion: v1\nkind: Pod\nmetadata:\n  name: with-node-affinity\nspec:\n  affinity:\n    nodeAffinity:\n      requiredDuringSchedulingIgnoredDuringExecution:\n        nodeSelectorTerms:\n        - matchExpressions:\n          - key: kubernetes.io/e2e-az-name\n            operator: In\n            values:\n            - e2e-az1\n            - e2e-az2\n      preferredDuringSchedulingIgnoredDuringExecution:\n      - weight: 1\n        preference:\n          matchExpressions:\n          - key: another-node-label-key\n            operator: In\n            values:\n            - another-node-label-value\n  containers:\n  - name: with-node-affinity\n    image: k8s.gcr.io/pause:2.0\n")
        .resolutionAndExample('Check pod affinity and anti-affinity', "\n# Pod Affinity and Anti-Affinity\napiVersion: v1\nkind: Pod\nmetadata:\n  name: with-pod-affinity\nspec:\n  affinity:\n    podAffinity:\n      requiredDuringSchedulingIgnoredDuringExecution:\n      - labelSelector:\n          matchExpressions:\n          - key: security\n            operator: In\n            values:\n            - S1\n        topologyKey: topology.kubernetes.io/zone\n    podAntiAffinity:\n      preferredDuringSchedulingIgnoredDuringExecution:\n      - weight: 100\n        podAffinityTerm:\n          labelSelector:\n            matchExpressions:\n            - key: security\n              operator: In\n              values:\n              - S2\n          topologyKey: topology.kubernetes.io/zone\n  containers:\n  - name: with-pod-affinity\n    image: k8s.gcr.io/pause:2.0\n")
        .resolutionAndExample('Check PodOverhead feature', "\n# PodOverhead\n---\nkind: RuntimeClass\napiVersion: node.k8s.io/v1\nmetadata:\n  name: kata-fc\nhandler: kata-fc\noverhead:\n  podFixed:\n    memory: \"120Mi\"\n    cpu: \"250m\"\n\n---\napiVersion: v1\nkind: Pod\nmetadata:\n  name: test-pod\nspec:\n  runtimeClassName: kata-fc\n  containers:\n  - name: busybox-ctr\n    image: busybox\n    stdin: true\n    tty: true\n    resources:\n      limits:\n        cpu: 500m\n        memory: 100Mi\n  - name: nginx-ctr\n    image: nginx\n    resources:\n      limits:\n        cpu: 1500m\n        memory: 100Mi\n")
        .resolutionAndExample('Check Node Taints and Pod Tolerations feature', "\n# Taints and Tolerations\napiVersion: v1\nkind: Pod\nmetadata:\n  name: nginx\n  labels:\n    env: test\nspec:\n  containers:\n  - name: nginx\n    image: nginx\n    imagePullPolicy: IfNotPresent\n  tolerations:\n  - key: \"example-key\"\n    operator: \"Exists\"\n    effect: \"NoSchedule\"\n")
        .externalLink('https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle/#pod-conditions')
        .externalLink('https://kubernetes.io/docs/concepts/scheduling-eviction/')
        .externalLink('https://kubernetes.io/docs/concepts/scheduling-eviction/assign-pod-node/')
        .externalLink('https://kubernetes.io/docs/tasks/debug-application-cluster/debug-application/')
        .externalLink('https://kubernetes.io/docs/reference/access-authn-authz/admission-controllers/#noderestriction')
        .externalLink('https://kubernetes.io/docs/reference/kubernetes-api/workload-resources/pod-v1/#PodStatus');
}
exports.setupPOD_STATUS_NOT_SCHEDULED = setupPOD_STATUS_NOT_SCHEDULED;
//# sourceMappingURL=POD_STATUS_NOT_SCHEDULED.js.map