"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RuleProcessor = void 0;
var the_lodash_1 = __importDefault(require("the-lodash"));
var the_promise_1 = require("the-promise");
var hash_utils_1 = require("../../utils/hash-utils");
var processor_1 = require("../target/processor");
var processor_2 = require("../validator/processor");
var execution_state_1 = require("../execution-state");
var RuleProcessor = /** @class */ (function () {
    function RuleProcessor(state, rule) {
        this._messageHashes = null;
        this._state = state;
        this._ruleTargetSrc = rule.target;
        this._ruleScriptSrc = rule.script;
        this._executeResult = null;
        this._messageHashes = null;
        this._executionState = new execution_state_1.ExecutionState(state);
    }
    RuleProcessor.prototype.process = function () {
        var _this = this;
        this._executeResult = {
            success: true,
            targetItems: [],
            messages: [],
            ruleItems: {},
        };
        this._messageHashes = {};
        return this._prepare()
            .then(function () {
            if (!_this._hasError()) {
                return;
            }
            return _this._execute();
        })
            .then(function () {
            return _this._getExecuteResult();
        })
            .catch(function () {
            _this._acceptScriptErrors('rule', {
                success: false,
                messages: ['Unknown error happened.'],
            });
            return _this._getExecuteResult();
        });
    };
    RuleProcessor.prototype._prepare = function () {
        var _this = this;
        return Promise.resolve()
            .then(function () { return _this._prepareTargets(); })
            .then(function () { return _this._prepareScript(); });
    };
    RuleProcessor.prototype._execute = function () {
        var _this = this;
        return this._executeTarget().then(function () {
            if (!_this._hasError()) {
                return;
            }
            return _this._executeValidators();
        });
    };
    RuleProcessor.prototype._prepareTargets = function () {
        var _this = this;
        this._targetProcessor = new processor_1.TargetProcessor(this._ruleTargetSrc, this._executionState);
        return this._targetProcessor.prepare().then(function (result) {
            // console.log("[RULE-PROCESSOR] TARGETS PREPARE RESULT: ", result)
            _this._acceptScriptErrors('target', result);
        });
    };
    RuleProcessor.prototype._prepareScript = function () {
        var _this = this;
        this._validationProcessor = new processor_2.ValidationProcessor(this._ruleScriptSrc, this._executionState);
        return this._validationProcessor.prepare().then(function (result) {
            // console.log("[RULE-PROCESSOR] SCRIPT PREPARE RESULT: ", result)
            _this._acceptScriptErrors('script', result);
        });
    };
    RuleProcessor.prototype._executeTarget = function () {
        var _this = this;
        return this._targetProcessor.execute().then(function (result) {
            _this._acceptScriptErrors('target', result);
            if (result.success) {
                _this._executeResult.targetItems = result.items;
            }
        });
    };
    RuleProcessor.prototype._executeValidators = function () {
        var _this = this;
        return the_promise_1.MyPromise.serial(this._executeResult.targetItems, function (x) {
            return _this._executeValidator(x);
        });
    };
    RuleProcessor.prototype._executeValidator = function (item) {
        var _this = this;
        return this._validationProcessor.execute(item.dn, this._state).then(function (result) {
            _this._acceptScriptErrors('script', result);
            if (result.success) {
                if (result.validation.hasErrors) {
                    _this._getRuleItem(item.dn).errors = {
                        present: true,
                        messages: the_lodash_1.default.keys(result.validation.errorMsgs),
                    };
                }
                if (result.validation.hasWarnings) {
                    _this._getRuleItem(item.dn).warnings = {
                        present: true,
                        messages: the_lodash_1.default.keys(result.validation.warnMsgs),
                    };
                }
                if (result.validation.marks) {
                    var marksList = the_lodash_1.default.keys(result.validation.marks);
                    if (marksList.length > 0) {
                        _this._getRuleItem(item.dn).marks = marksList;
                    }
                }
            }
        });
    };
    RuleProcessor.prototype._getRuleItem = function (dn) {
        if (!this._executeResult.ruleItems[dn]) {
            this._executeResult.ruleItems[dn] = {};
        }
        return this._executeResult.ruleItems[dn];
    };
    RuleProcessor.prototype._hasError = function () {
        return this._executeResult.success;
    };
    RuleProcessor.prototype._acceptScriptErrors = function (source, result) {
        if (!result.success) {
            this._executeResult.success = false;
            for (var _i = 0, _a = result.messages; _i < _a.length; _i++) {
                var msg = _a[_i];
                var msgInfo = {
                    source: [source],
                    msg: msg,
                };
                var hash = (0, hash_utils_1.calculateObjectHashStr)(msgInfo);
                if (!(hash in this._messageHashes)) {
                    this._messageHashes[hash] = true;
                    this._executeResult.messages.push(msgInfo);
                }
            }
        }
    };
    RuleProcessor.prototype._getExecuteResult = function () {
        var result = this._executeResult;
        this._executeResult = null;
        return result;
    };
    return RuleProcessor;
}());
exports.RuleProcessor = RuleProcessor;
//# sourceMappingURL=processor.js.map