"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TargetProcessor = void 0;
var compiler_1 = require("../compiler");
var scope_1 = require("../../spec/target/scope");
var scope_builder_1 = require("./scope-builder");
var fetcher_1 = require("../query/fetcher");
var TargetProcessor = /** @class */ (function () {
    function TargetProcessor(src, executionState, scopeBuilderExecutor) {
        this._src = src;
        this._executionState = executionState;
        this._scope = new scope_1.Scope();
        this._errorMessages = [];
        this._scopeBuilderExecutor = scopeBuilderExecutor !== null && scopeBuilderExecutor !== void 0 ? scopeBuilderExecutor : scope_builder_1.makeTargetRootScope;
    }
    Object.defineProperty(TargetProcessor.prototype, "scope", {
        get: function () {
            return this._scope;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TargetProcessor.prototype, "k8sApiResources", {
        get: function () {
            return this._executionState.k8sApiResources;
        },
        enumerable: false,
        configurable: true
    });
    TargetProcessor.prototype.prepare = function () {
        var _this = this;
        this._errorMessages = [];
        var result = {
            success: false,
            messages: this._errorMessages,
        };
        return this._loadModule()
            .then(function (runnable) { return runnable.run(); })
            .then(function () {
            _this._scope.finalize();
        })
            .then(function () {
            _this._validate();
            result.success = result.messages.length == 0;
        })
            .catch(function (reason) {
            result.success = false;
            _this._addError(reason.message);
        })
            .then(function () { return result; });
    };
    TargetProcessor.prototype.execute = function () {
        var _this = this;
        return Promise.resolve()
            .then(function () {
            var fetcher = new fetcher_1.QueryFetcher(_this._executionState, _this._scope);
            var result = fetcher.execute();
            return result;
        });
    };
    TargetProcessor.prototype._loadModule = function () {
        var _this = this;
        var rootScope = {};
        var rootScopeBuilder = {
            setup: function (name, func) {
                rootScope[name] = func;
            }
        };
        return Promise.resolve().then(function () {
            _this._scopeBuilderExecutor(rootScopeBuilder, _this._scope, _this._executionState);
            var compiler = new compiler_1.Compiler(_this._src, 'RULE_TARGET', rootScope);
            return compiler.compile();
        });
    };
    TargetProcessor.prototype._validate = function () {
        if (this._scope._chain.length == 0) {
            this._addError('No target specified.');
            return;
        }
        this._validateScope(this._scope);
    };
    TargetProcessor.prototype._validateScope = function (scope) {
        if (this._scope._chain.length > 1) {
            this._addError('Fanout targets are not supported.');
            return;
        }
        for (var _i = 0, _a = scope._chain; _i < _a.length; _i++) {
            var targetSelector = _a[_i];
            this._validateTargetSelector(targetSelector);
        }
    };
    TargetProcessor.prototype._validateTargetSelector = function (targetSelector) {
        this._validateScope(targetSelector._scope);
    };
    TargetProcessor.prototype._addError = function (msg) {
        // console.log("[TARGET-PROCESSOR] ERROR: ", msg)
        this._errorMessages.push(msg);
    };
    return TargetProcessor;
}());
exports.TargetProcessor = TargetProcessor;
//# sourceMappingURL=processor.js.map