"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClusterInfoFetcher = void 0;
var the_lodash_1 = __importDefault(require("the-lodash"));
var the_promise_1 = require("the-promise");
var utils_1 = require("./utils");
var ClusterInfoFetcher = /** @class */ (function () {
    function ClusterInfoFetcher(logger, client) {
        this._rootApiVersion = null;
        this._apiGroups = {};
        this._selectedApiGroups = [];
        this._enabledApiGroups = {};
        this._preferredVersions = {};
        this.logger = logger;
        this._client = client;
    }
    ClusterInfoFetcher.prototype.perform = function () {
        var _this = this;
        return Promise.resolve()
            .then(function () { return _this._discoverRootApi(); })
            .then(function () { return _this._fetchApiGroup(null, _this._rootApiVersion, false); })
            .then(function () { return _this._discoverApiGroups(); })
            .then(function (apis) {
            return the_promise_1.MyPromise.parallel(apis, function (x) { return _this._fetchApiGroup(x.name, x.version, true); });
        })
            .then(function () { return _this._finalizeApis(); })
            .then(function () {
            var info = {
                rootApiVersion: _this._rootApiVersion,
                apiGroups: _this._selectedApiGroups,
                enabledApiGroups: _this._enabledApiGroups,
                preferredVersions: _this._preferredVersions
            };
            return info;
        });
    };
    ClusterInfoFetcher.prototype._discoverRootApi = function () {
        var _this = this;
        return this._client.request('GET', '/api')
            .then(function (result) {
            _this.logger.debug("[discoverRootApi] ", result);
            var version = the_lodash_1.default.last(result.versions);
            if (!version) {
                _this.logger.error("[discoverRootApi] Failed to determine root api version", result);
                throw new Error('Failed to determine root api version');
                return;
            }
            _this._rootApiVersion = version;
            _this.logger.info("[discoverRootApi] root version: %s", _this._rootApiVersion);
        });
    };
    ClusterInfoFetcher.prototype._discoverApiGroups = function () {
        var _this = this;
        return this._client.request('GET', '/apis')
            .then(function (result) {
            var _a;
            _this.logger.silly("[_discoverApiGroups] ", result);
            var apis = [];
            for (var _i = 0, _b = result.groups; _i < _b.length; _i++) {
                var groupInfo = _b[_i];
                var preferredVersion = (_a = groupInfo.preferredVersion) === null || _a === void 0 ? void 0 : _a.version;
                if (preferredVersion) {
                    _this._preferredVersions[groupInfo.name] = preferredVersion;
                }
                _this.logger.info("[discoverApiGroups] %s (%s). All versions: [%s]", groupInfo.name, preferredVersion !== null && preferredVersion !== void 0 ? preferredVersion : "?", groupInfo.versions.map(function (x) { return x.version; }).join(', '));
                for (var _c = 0, _d = groupInfo.versions; _c < _d.length; _c++) {
                    var versionInfo = _d[_c];
                    apis.push({
                        name: groupInfo.name,
                        version: versionInfo.version
                    });
                }
            }
            return apis;
        });
    };
    ClusterInfoFetcher.prototype._fetchApiGroup = function (group, version, allowError) {
        var _this = this;
        this.logger.info("[_fetchApiGroup] group: %s, version: %s", group, version);
        var url;
        if (group) {
            url = "/apis/".concat(group, "/").concat(version);
        }
        else {
            url = "/api/".concat(version);
        }
        return this._client.request('GET', url)
            .catch(function (reason) {
            _this.logger.error("Error fetching api group: %s :: %s", group, version);
            if (allowError) {
                return {
                    groupVersion: '',
                    resources: []
                };
            }
            throw reason;
        })
            .then(function (result) {
            if (!(result === null || result === void 0 ? void 0 : result.resources)) {
                return;
            }
            for (var _i = 0, _a = result.resources; _i < _a.length; _i++) {
                var resource = _a[_i];
                var nameParts = resource.name.split('/');
                // this.logger.info("[fetchApiGroup] nameParts.name :: ", resource.name, nameParts);
                if (nameParts.length == 1) {
                    // EXPLANATION: This is to skip "Status" objects
                    _this.logger.silly("[_fetchApiGroup] ", resource);
                    _this._setupApiGroup(resource.kind, group, version, nameParts[0], resource.namespaced, resource.verbs);
                }
            }
        });
    };
    ClusterInfoFetcher.prototype._setupApiGroup = function (kindName, apiName, version, pluralName, isNamespaced, verbs) {
        if (!verbs) {
            this.logger.warn("[_setupApiGroup] missing verbs for apiName: %s, kindName: %s, version: %s, namespaced: %s", apiName, kindName, version, isNamespaced);
        }
        verbs = verbs || [];
        this.logger.info("[_setupApiGroup] apiName: %s, kindName: %s, version: %s, namespaced: %s, verbs: [%s]", apiName, kindName, version, isNamespaced, verbs.join(", "));
        var id = (0, utils_1.apiId)(kindName, apiName);
        var apiVersion = apiName ? "".concat(apiName, "/").concat(version) : version;
        var apiGroupInfo = {
            id: id,
            apiVersion: apiVersion,
            apiName: apiName,
            version: version,
            kindName: kindName,
            pluralName: pluralName,
            isNamespaced: isNamespaced,
            verbs: verbs,
            isEnabled: true,
            allVersions: []
        };
        this.logger.silly("[_setupApiGroup] %s => ", id, apiGroupInfo);
        if (!this._apiGroups[id]) {
            this._apiGroups[id] = {
                kindName: kindName,
                apiName: apiName,
                versions: []
            };
        }
        this._apiGroups[id].versions.push(apiGroupInfo);
    };
    ClusterInfoFetcher.prototype._finalizeApis = function () {
        // this.logger.silly("[_finalizeApis] ", this._apiGroups);
        for (var _i = 0, _a = the_lodash_1.default.values(this._apiGroups); _i < _a.length; _i++) {
            var apiGroupVersions = _a[_i];
            var selectedApiGroup = this._selectApiGroupVersion(apiGroupVersions);
            selectedApiGroup.allVersions = apiGroupVersions.versions.map(function (x) { return x.version; });
            this._selectedApiGroups.push(selectedApiGroup);
            this._finalizeApi(selectedApiGroup);
        }
    };
    ClusterInfoFetcher.prototype._selectApiGroupVersion = function (apiGroupVersions) {
        var preferredVersion = this._getPreferredVersion(apiGroupVersions.apiName);
        if (preferredVersion) {
            var preferredGroup = the_lodash_1.default.find(apiGroupVersions.versions, function (x) { return x.version === preferredVersion; });
            if (preferredGroup) {
                return preferredGroup;
            }
        }
        var orderedVersions = the_lodash_1.default.orderBy(apiGroupVersions.versions, function (x) { return x.version; });
        return the_lodash_1.default.head(orderedVersions);
    };
    ClusterInfoFetcher.prototype._getPreferredVersion = function (apiName) {
        if (apiName) {
            return this._preferredVersions[apiName];
        }
        else {
            return this._rootApiVersion;
        }
    };
    ClusterInfoFetcher.prototype._finalizeApi = function (apiGroupInfo) {
        if (this._isApiDisabled(apiGroupInfo)) {
            this.logger.info("[_finalizeApi] Skipping. Resource: %s", apiGroupInfo.id);
            apiGroupInfo.isEnabled = false;
            return;
        }
        this.logger.debug("[_finalizeApi] Setup. Resource: %s :: %s...", apiGroupInfo.id, apiGroupInfo.version);
        this._enabledApiGroups[apiGroupInfo.id] = apiGroupInfo;
    };
    ClusterInfoFetcher.prototype._isApiDisabled = function (apiGroupInfo) {
        return false;
    };
    return ClusterInfoFetcher;
}());
exports.ClusterInfoFetcher = ClusterInfoFetcher;
//# sourceMappingURL=cluster-info-fetcher.js.map