"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseOpenApiDocumentParser = void 0;
var the_lodash_1 = __importDefault(require("the-lodash"));
var BaseOpenApiDocumentParser = /** @class */ (function () {
    function BaseOpenApiDocumentParser(logger, k8sApiData, documentSchemas) {
        this._convertedTargetRefs = {};
        this._logger = logger;
        this._k8sApiData = k8sApiData;
        this._documentSchemas = documentSchemas !== null && documentSchemas !== void 0 ? documentSchemas : {};
    }
    BaseOpenApiDocumentParser.prototype._convertApiResource = function (path, apiResource, resourceRef) {
        this._logger.debug("[_convertApiResource] ApiResource-V3: %s", path);
        this._logger.silly("[_convertApiResource] ApiResource-V3: %s => %s ::", path, resourceRef, apiResource);
        var isNamespaced = the_lodash_1.default.includes(path, 'namespaces/{namespace}');
        this._convertReference(resourceRef);
        var resourceInfo = {
            definitionId: this._getResourceKeyFromRef(resourceRef),
            namespaced: isNamespaced
        };
        this._k8sApiData.resources[the_lodash_1.default.stableStringify(apiResource)] = resourceInfo;
    };
    BaseOpenApiDocumentParser.prototype._convertReference = function (origRef) {
        // this._logger.silly("[_convertReference] %s", origRef);
        var targetRef = this._convertRef(origRef);
        if (this._convertedTargetRefs[targetRef]) {
            return targetRef;
        }
        this._convertedTargetRefs[targetRef] = true;
        var resourceKey = this._getResourceKeyFromRef(origRef);
        var openApiSchema = this._documentSchemas[resourceKey];
        if (!openApiSchema) {
            this._logger.error("[_convertOpenApiV3Document] Missing %s", resourceKey);
            throw new Error("Missing reference: ".concat(origRef));
        }
        var schema = this._convertSchema(openApiSchema);
        this._k8sApiData.definitions[resourceKey] = schema;
        return targetRef;
    };
    BaseOpenApiDocumentParser.prototype._convertSchema = function (openApiSchema) {
        var refValue = openApiSchema['$ref'];
        if (refValue) {
            var newRef = this._convertReference(refValue);
            return {
                '$ref': newRef
            };
        }
        var customFix = this._applyCustomFix(openApiSchema);
        if (customFix) {
            return customFix;
        }
        var schema = {
            type: openApiSchema.type,
            format: openApiSchema.format
        };
        if (openApiSchema.properties) {
            schema.properties = this._convertProperties(openApiSchema.properties);
        }
        if (openApiSchema.items) {
            schema.items = this._convertSchema(openApiSchema.items);
        }
        if (openApiSchema.allOf) {
            schema.allOf = this._convertArray(openApiSchema.allOf);
        }
        if (openApiSchema.oneOf) {
            schema.oneOf = this._convertArray(openApiSchema.oneOf);
        }
        if (openApiSchema.anyOf) {
            schema.anyOf = this._convertArray(openApiSchema.anyOf);
        }
        if (the_lodash_1.default.isNotNullOrUndefined(openApiSchema.default)) {
            schema.default = openApiSchema.default;
        }
        if (the_lodash_1.default.isNotNullOrUndefined(openApiSchema.required)) {
            schema.required = openApiSchema.required;
        }
        if (the_lodash_1.default.isNotNullOrUndefined(openApiSchema.enum)) {
            // BUG FIX: https://github.com/kubevious/cli/issues/21
            schema.enum = the_lodash_1.default.uniq(openApiSchema.enum);
        }
        // BUG FIX: https://github.com/kubevious/cli/issues/13
        if (!schema.type) {
            if (schema.format) {
                if (!schema.allOf && !schema.oneOf && !schema.anyOf) {
                    schema.type = "string";
                }
            }
        }
        if (openApiSchema.type === "object") {
            schema.additionalProperties = false;
            if (the_lodash_1.default.isNotNullOrUndefined(openApiSchema.additionalProperties)) {
                if (the_lodash_1.default.isBoolean(openApiSchema.additionalProperties)) {
                    schema.additionalProperties = openApiSchema.additionalProperties;
                }
                else {
                    schema.additionalProperties = this._convertSchema(openApiSchema.additionalProperties);
                }
            }
            else {
                if (openApiSchema['x-kubernetes-preserve-unknown-fields']) {
                    schema.additionalProperties = true;
                }
                else {
                    if (the_lodash_1.default.isNotNullOrUndefined(openApiSchema.additionalProperties)) {
                        if (openApiSchema.additionalProperties['x-kubernetes-preserve-unknown-fields']) {
                            // NOTE: Not the right way, but used in Traefik:
                            // https://raw.githubusercontent.com/traefik/traefik/v2.9/docs/content/reference/dynamic-configuration/kubernetes-crd-definition-v1.yml
                            schema.additionalProperties = true;
                        }
                    }
                }
            }
        }
        return schema;
    };
    BaseOpenApiDocumentParser.prototype._convertProperties = function (properties) {
        var converted = {};
        for (var _i = 0, _a = the_lodash_1.default.keys(properties); _i < _a.length; _i++) {
            var name_1 = _a[_i];
            converted[name_1] = this._convertSchema(properties[name_1]);
        }
        return converted;
    };
    BaseOpenApiDocumentParser.prototype._convertArray = function (openApiSchemas) {
        var _this = this;
        return openApiSchemas.map(function (x) { return _this._convertSchema(x); });
    };
    BaseOpenApiDocumentParser.prototype._applyCustomFix = function (openApiSchema) {
        if (openApiSchema.format === 'int-or-string') {
            return {
                oneOf: [
                    { type: "string" },
                    { type: "integer" }
                ]
            };
        }
        return null;
    };
    BaseOpenApiDocumentParser.prototype._convertRef = function (origRef) {
        var resourceKey = this._getResourceKeyFromRef(origRef);
        return "#/definitions/".concat(resourceKey);
    };
    BaseOpenApiDocumentParser.prototype._getResourceKeyFromRef = function (origRef) {
        var index = origRef.lastIndexOf('/');
        return origRef.substring(index + 1);
    };
    return BaseOpenApiDocumentParser;
}());
exports.BaseOpenApiDocumentParser = BaseOpenApiDocumentParser;
//# sourceMappingURL=base-open-api-document-parser.js.map