"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResourceAccessor = void 0;
var the_lodash_1 = __importDefault(require("the-lodash"));
var uuid_1 = require("uuid");
var resource_watch_1 = require("./resource-watch");
var ResourceAccessor = /** @class */ (function () {
    function ResourceAccessor(parent, apiName, version, pluralName, kindName) {
        this._watches = {};
        this._parent = parent;
        this._logger = parent.logger;
        this._apiName = apiName;
        this._version = version;
        this._kindName = kindName;
        this._pluralName = pluralName;
    }
    Object.defineProperty(ResourceAccessor.prototype, "logger", {
        get: function () {
            return this._logger;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ResourceAccessor.prototype, "apiName", {
        get: function () {
            return this._apiName;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ResourceAccessor.prototype, "kindName", {
        get: function () {
            return this._kindName;
        },
        enumerable: false,
        configurable: true
    });
    ResourceAccessor.prototype.close = function () {
        for (var _i = 0, _a = the_lodash_1.default.values(this._watches); _i < _a.length; _i++) {
            var watch = _a[_i];
            watch.close();
        }
        this._watches = {};
    };
    ResourceAccessor.prototype.queryAll = function (namespace, labelFilter) {
        var _this = this;
        var uriParts = this._makeUriParts(namespace);
        return this._getRequest(uriParts, { labelSelector: labelFilter })
            .then(function (result) {
            return _this._formatQueryAllResult(result);
        });
    };
    ResourceAccessor.prototype.queryAllSync = function (namespace, labelFilter) {
        var uriParts = this._makeUriParts(namespace);
        var result = this._getRequestSync(uriParts, { labelSelector: labelFilter });
        return this._formatQueryAllResult(result);
    };
    ResourceAccessor.prototype._formatQueryAllResult = function (result) {
        if (!result) {
            return [];
        }
        for (var _i = 0, _a = result.items; _i < _a.length; _i++) {
            var x = _a[_i];
            if (!x.kind) {
                x.kind = this.kindName;
            }
            if (!x.apiVersion) {
                x.apiVersion = result.apiVersion;
            }
        }
        return result.items;
    };
    ResourceAccessor.prototype.watchAll = function (namespace, cb, connectCb, disconnectCb) {
        var _this = this;
        var id = (0, uuid_1.v4)();
        var scope = {
            request: this._parent.request.bind(this._parent)
        };
        var watch = new resource_watch_1.ResourceWatch(this._logger.sublogger("K8sResourceWatch"), this, namespace, cb, connectCb, disconnectCb, function () {
            delete _this._watches[id];
        }, scope);
        this._watches[id] = watch;
        watch.start();
        return watch;
    };
    ResourceAccessor.prototype.query = function (namespace, name) {
        var uriParts = this._makeUriParts(namespace);
        uriParts.push(name);
        return this._getRequest(uriParts);
    };
    ResourceAccessor.prototype.querySync = function (namespace, name) {
        var uriParts = this._makeUriParts(namespace);
        uriParts.push(name);
        return this._getRequestSync(uriParts);
    };
    ResourceAccessor.prototype.create = function (namespace, body) {
        var uriParts = this._makeUriParts(namespace);
        var newBody = this._setupBody(body);
        return this._postRequest(uriParts, {}, newBody);
    };
    ResourceAccessor.prototype.delete = function (namespace, name) {
        var uriParts = this._makeUriParts(namespace);
        uriParts.push(name);
        return this._deleteRequest(uriParts, {});
    };
    ResourceAccessor.prototype.update = function (namespace, name, body) {
        var uriParts = this._makeUriParts(namespace);
        uriParts.push(name);
        var newBody = this._setupBody(body);
        return this._putRequest(uriParts, {}, newBody);
    };
    ResourceAccessor.prototype._getRequest = function (uriParts, params) {
        var url = this._joinUrls(uriParts);
        this.logger.info('[_getRequest] %s', url);
        params = this._makeGetParams(params);
        return this._parent.request('GET', url, params);
    };
    ResourceAccessor.prototype._makeGetParams = function (params) {
        if (params) {
            params = the_lodash_1.default.clone(params);
            if (params.labelSelector) {
                if (the_lodash_1.default.keys(params.labelSelector).length == 0) {
                    delete params.labelSelector;
                }
                var selectorParts = the_lodash_1.default.keys(params.labelSelector).map(function (x) { return x + '=' + params.labelSelector[x]; });
                params.labelSelector = selectorParts.join(',');
            }
        }
        return params;
    };
    ResourceAccessor.prototype._getRequestSync = function (uriParts, params) {
        var url = this._joinUrls(uriParts);
        this.logger.info('[_getRequestSync] %s', url);
        params = this._makeGetParams(params);
        return this._parent.requestSync('GET', url, params);
    };
    ResourceAccessor.prototype._postRequest = function (uriParts, params, body) {
        var url = this._joinUrls(uriParts);
        this.logger.info('[_postRequest] %s', url);
        return this._parent.request('POST', url, params, body);
    };
    ResourceAccessor.prototype._deleteRequest = function (uriParts, params) {
        var url = this._joinUrls(uriParts);
        this.logger.info('[_deleteRequest] %s', url);
        return this._parent.request('DELETE', url, params);
    };
    ResourceAccessor.prototype._putRequest = function (uriParts, params, body) {
        var url = this._joinUrls(uriParts);
        this.logger.info('[_putRequest] %s', url);
        return this._parent.request('PUT', url, params, body);
    };
    ResourceAccessor.prototype._setupBody = function (body) {
        var newBody = the_lodash_1.default.clone(body);
        newBody.kind = this._kindName;
        if (this._apiName) {
            newBody.apiVersion = this._apiName + '/' + this._version;
        }
        else {
            newBody.apiVersion = this._version;
        }
        return newBody;
    };
    ResourceAccessor.prototype._joinUrls = function (uriParts) {
        var prefixParts;
        if (the_lodash_1.default.isNullOrUndefined(this._apiName)) {
            prefixParts = ['', 'api', this._version];
        }
        else {
            prefixParts = ['', 'apis', this._apiName, this._version];
        }
        var newUriParts = the_lodash_1.default.concat(prefixParts, uriParts);
        var url = newUriParts.join('/');
        return url;
    };
    ResourceAccessor.prototype._makeUriParts = function (namespace) {
        var uriParts = [];
        // if (watch) {
        //     uriParts.push('watch')
        // }
        if (namespace) {
            uriParts.push('namespaces');
            uriParts.push(namespace);
        }
        uriParts.push(this._pluralName);
        return uriParts;
    };
    return ResourceAccessor;
}());
exports.ResourceAccessor = ResourceAccessor;
//# sourceMappingURL=resource-accessor.js.map