"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Executor = void 0;
var the_lodash_1 = __importDefault(require("the-lodash"));
var promise_1 = require("./promise");
var Executor = /** @class */ (function () {
    function Executor(items, action, options) {
        this.origItems = [];
        this.items = [];
        this.processing = {};
        this.results = [];
        this.isProcessingFailed = false;
        this.waitingCoolDown = false;
        for (var i = 0; i < items.length; i++) {
            var itemInfo = {
                item: items[i],
                index: i,
                errors: 0,
                delay: null,
            };
            this.origItems.push(itemInfo);
            this.items.push(itemInfo);
        }
        this.action = action;
        this.options = options;
    }
    Object.defineProperty(Executor.prototype, "unfinishedCount", {
        get: function () {
            return the_lodash_1.default.keys(this.processing).length;
        },
        enumerable: false,
        configurable: true
    });
    Executor.prototype.execute = function () {
        var _this = this;
        return promise_1.MyPromise.construct(function (resolve, reject) {
            _this.rootResolve = resolve;
            _this.rootReject = reject;
            _this._tryProcess();
        });
    };
    Executor.prototype._tryProcess = function () {
        var _this = this;
        var _a;
        if (this.isProcessingFailed) {
            if (this.options.waitCompleteBeforeExit) {
                if (this.unfinishedCount === 0) {
                    this._finishFailure();
                }
            }
            else {
                this._finishFailure();
            }
            return;
        }
        if (this.waitingCoolDown) {
            if (this.unfinishedCount === 0) {
                this.waitingCoolDown = false;
            }
            else {
                return;
            }
        }
        if (this.unfinishedCount >= this.options.concurrency) {
            return;
        }
        var now = new Date();
        for (var i = 0; i < this.items.length; i++) {
            var itemInfo = this.items[i];
            if (this._canProcessItem(now, itemInfo)) {
                this.items = __spreadArray(__spreadArray([], this.items.slice(0, i), true), this.items.slice(i + 1), true);
                this._processItem(itemInfo);
                return;
            }
        }
        if (this.items.length > 0) {
            var pauseMs = (_a = this.pauseTillNextTry) !== null && _a !== void 0 ? _a : 500;
            promise_1.MyPromise.timeout(pauseMs).then(function () {
                _this._tryProcess();
                return null;
            });
            return;
        }
        if (this.unfinishedCount === 0) {
            this._finishSuccess();
        }
    };
    Executor.prototype._canProcessItem = function (now, itemInfo) {
        if (!itemInfo.lastFailureMoment) {
            return true;
        }
        if (itemInfo.delay) {
            var diff = now.valueOf() - itemInfo.lastFailureMoment.valueOf();
            if (diff >= itemInfo.delay) {
                return true;
            }
            else {
                if (!this.pauseTillNextTry) {
                    this.pauseTillNextTry = diff;
                }
                else {
                    this.pauseTillNextTry = Math.min(this.pauseTillNextTry, diff);
                }
            }
        }
        else {
            return true;
        }
        return false;
    };
    Executor.prototype._processItem = function (itemInfo) {
        var _this = this;
        this.processing[itemInfo.index] = itemInfo;
        try {
            var actionResult = this.action(itemInfo.item);
            Promise.resolve(actionResult)
                .then(function (result) {
                _this._handleItemFinished(itemInfo, result);
                return null;
            })
                .catch(function (reason) {
                _this._handleItemFailed(itemInfo, reason);
                return null;
            });
        }
        catch (reason) {
            this._handleItemFailed(itemInfo, reason);
        }
        this._tryProcess();
    };
    Executor.prototype._handleItemFinished = function (itemInfo, result) {
        delete this.processing[itemInfo.index];
        this.results[itemInfo.index] = result;
        this._tryProcess();
    };
    Executor.prototype._handleItemFailed = function (itemInfo, reason) {
        delete this.processing[itemInfo.index];
        itemInfo.errors++;
        itemInfo.lastError = reason;
        if (this.options.unlimitedRetries || (itemInfo.errors <= this.options.retryCount)) {
            if (itemInfo.delay === null) {
                itemInfo.delay = this.options.initRetryDelay;
            }
            else {
                itemInfo.delay = Math.min(this.options.maxRetryDelay, itemInfo.delay * this.options.retryDelayCoeff);
            }
            itemInfo.lastFailureMoment = new Date();
            this.items.push(itemInfo);
            if (this.options.coolDownOnFailure) {
                this.waitingCoolDown = true;
            }
        }
        else {
            this.isProcessingFailed = true;
        }
        this._tryProcess();
    };
    Executor.prototype._finishSuccess = function () {
        if (!this.rootResolve) {
            return;
        }
        var rootResolve = this.rootResolve;
        this._finish();
        rootResolve(this.results);
    };
    Executor.prototype._finishFailure = function (error) {
        if (!this.rootReject) {
            return;
        }
        var rootReject = this.rootReject;
        this._finish();
        var errors = this.origItems.map(function (x) { return x.lastError; }).filter(function (x) { return x; });
        rootReject(errors);
    };
    Executor.prototype._finish = function () {
        this.rootResolve = undefined;
        this.rootReject = undefined;
    };
    return Executor;
}());
exports.Executor = Executor;
//# sourceMappingURL=executor.js.map